@@ -1,7 +1,7 @@
 #!/usr/bin/python3
 # -*- coding: utf-8 -*-
 
-#  Copyright © 2012-2015  B. Clausius <barcc@gmx.de>
+#  Copyright © 2012-2017  B. Clausius <barcc@gmx.de>
 #
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -21,19 +21,24 @@ import sys, os
 sys.path.insert(0, '.')
 import pickletools
 from multiprocessing import Pool, cpu_count
-from collections import defaultdict, OrderedDict
+from collections import defaultdict
 from bisect import bisect_left
 from array import array
 from itertools import chain
 from math import sin, cos, pi
+from contextlib import suppress
 
-from pybiklib.debug import DEBUG, DEBUG_MSG, DEBUG_NOLABEL, DEBUG_NOBEVEL, DEBUG_INVISIBLE, DEBUG_INDEXONLY
-from pybiklib.modelcommon import epsilon, filebyteorder, get_texcoords_range
+from pybiklib.debug import DEBUG_MSG, DEBUG_INDEXONLY, DEBUG_MODELFAST, DEBUG_MAXSIZE5
+from pybiklib.utils import epsilon, filebyteorder, get_texcoords_range
 # executable script needs absolute imports
 from buildlib.geom import roundeps, Vector
 from buildlib.modeldef import modeldefs
 
 
+minsize = 1
+maxsize = 10
+if DEBUG_MAXSIZE5:  maxsize = 5
+
 dumps = None
 
 def make_dumps(reproducible):
@@ -51,7 +56,7 @@ def make_dumps(reproducible):
                 else:   # proto 0 -- can't use EMPTY_DICT
                     self.write(pickle.MARK + pickle.DICT)
                 self.memoize(obj)
-                self._batch_setitems(sorted(obj.items()))
+                self._batch_setitems(sorted(obj.items(), key=lambda v: str(v)))
                 
             pickle._Pickler.dispatch[dict] = save_dict
             
@@ -79,34 +84,42 @@ def equal_vector_fuzzy(v1, v2):
     
     
 class VectorData:
-    __slots__ = 'vectors indices sorted cnts dups toindex'.split()
+    __slots__ = 'vectors origs indices sorted cnts dups toindex'.split()
     
-    def __init__(self, fast):
+    def __init__(self):
         self.vectors = []
+        self.origs = {}
         self.indices = []
         self.sorted = []
         self.cnts = 0
         self.dups = 0
-        self.toindex = self.toindex_fast if fast else self.toindex_dedup
+        self.toindex = self.toindex_fast if DEBUG_MODELFAST else self.toindex_dedup
         
-    def toindex_dedup(self, vector):
-        vector = vector.rounded()
+    def toindex_dedup(self, ovector):
+        with suppress(KeyError):
+            return self.origs[ovector]
+        vector = ovector.rounded()
         vector = [(0. if v == 0. else v) for v in vector]  # 0.0 == -0.0
         self.cnts += 1
         si = bisect_left(self.sorted, vector)
         i = len(self.vectors)
         if si < i and self.sorted[si] == vector:
             self.dups += 1
-            return self.indices[si]
-        self.vectors.append(vector)
-        self.sorted.insert(si, vector)
-        self.indices.insert(si, i)
+            i = self.indices[si]
+        else:
+            self.vectors.append(vector)
+            self.sorted.insert(si, vector)
+            self.indices.insert(si, i)
+        self.origs[ovector] = i
         return i
         
-    def toindex_fast(self, vector):
-        vector = vector.rounded()
+    def toindex_fast(self, ovector):
+        with suppress(KeyError):
+            return self.origs[ovector]
+        vector = ovector.rounded()
         i = len(self.vectors)
         self.vectors.append(vector)
+        self.origs[ovector] = i
         return i
         
         
@@ -115,7 +128,7 @@ class ModelFactory:
             'name', 'mformat', 'sizenames', 'defaultsize',
             'symmetries', 'axes', 'symbols', 'symbolsI', 'faces', 'facekeys',
             'normal_rotation_symbols', 'rotation_symbols', 'rotation_matrices',
-            'face_permutations', 'default_rotation')
+            'face_permutations', 'default_rotation', 'reversepick', 'slicesmode')
         
     def __init__(self, modeldef):
         self.modeldef = modeldef()
@@ -124,6 +137,15 @@ class ModelFactory:
     def __getattr__(self, attrname):
         return getattr(self.modeldef, attrname)
         
+    def _getattr(self, attrname):
+        value = getattr(self, attrname)
+        if attrname == 'axes':
+            return tuple(tuple(v) for v in value)
+        return value
+        
+    def getattrs(self):
+        return {attr: self._getattr(attr) for attr in self.mtype_attributes}
+        
     @staticmethod
     def _matrix_equal(m1, m2):
         for line1, line2 in zip(m1, m2):
@@ -218,111 +240,122 @@ class ModelFactory:
                     assert False, 'not a permutation'
         return rotated_position
         
-    @staticmethod
-    def mark_invisible_faces_bevel(polys):
-        for cell in polys.cells:
-            for other in ['cut', 'bevel']:
-                for hf in cell.halffaces:
-                    if hf.face.type != 'bevel':
-                        continue
-                    other_removed = False
-                    for he in hf.halfedges:
-                        face_ = he.other.halfface.face
-                        if face_.type == 'face':
-                            break
-                        if face_.type == other:
-                            break
-                        if face_.type.endswith('_removed'):
-                            other_removed = True
-                    else:
-                        if other_removed:
-                            hf.face.type += '_removed'
-                            
     def get_data(self, sizes, vectordata):
-        polys, slice_centers = self.create_cells(sizes)
+        polys = self.create_cells(sizes)
         
-        normals_polys = OrderedDict()  # for reproducible build
-        for cell in polys.cells:
-            for hf in cell.halffaces:
-                key = (cell.indices, hf.face.id)
-                normals_polys[key] = self.normals[hf.face.id] if hf.face.type == 'face' else hf.normal()
         data_cells_visible_faces = [[f.id for f in cell.faces if f.type == 'face'] for cell in polys.cells]
-        normals = [self.normals[sym] for sym in self.faces]
         polys_verts = [v.point for v in polys.verts]
-        data_texranges_mosaic = [list(get_texcoords_range(polys_verts, normal)) for normal in normals]
-        data_normals = [vectordata.toindex(normal) for normal in normals]
+        data_texranges_mosaic = [list(get_texcoords_range(polys_verts, self.normals[sym])) for sym in self.faces]
         data_rotated_position = self.get_rotated_position(polys.cells)
         data_cell_indices = [cell.indices for cell in polys.cells]
+        data_cell_centers = [vectordata.toindex(cell.center()) for cell in polys.cells]
         data_pick_polygons = list(self.gl_pick_polygons(polys.cells, vectordata))
+        blocksdata = self.gl_block_data(polys, vectordata)
         
-        # beveled cells
-        normals_polys = {k: vectordata.toindex(n) for k, n in normals_polys.items()}
-        polys_beveled = polys
-        polys_beveled.bevel(self.bevel_width)
-        self.mark_invisible_faces_bevel(polys_beveled)
-        if DEBUG_INVISIBLE:
-            for face in polys_beveled.faces:
-                if face.type.endswith('_removed'):
-                    face.type = face.type.rsplit('_', 1)[0]
-                else:
-                    face.type += '_removed'
-        polys_beveled.remove_faces(lambda f: f.type.endswith('_removed'))
-        
-        data_block_polygons = [[list(self.gl_label_polygons(cell, vectordata)),
-                                list(self.gl_black_polygons(cell, normals_polys, vectordata))]
-                                    for cell in polys_beveled.cells]
         return {
                 'rotated_position': data_rotated_position,
                 'cell_indices': data_cell_indices,
+                'cell_centers': data_cell_centers,
                 'cells_visible_faces': data_cells_visible_faces,
                 'texranges_mosaic': data_texranges_mosaic,
-                'normals': data_normals,
-                'block_polygons': data_block_polygons,
-                'pick_polygons': data_pick_polygons,
-                'slice_centers': slice_centers,
+                'blocksdata': blocksdata,
+                'facesdata': data_pick_polygons,
             }
         
-    def gl_label_polygons(self, cell, vectordata):
-        if DEBUG:
-            if DEBUG_NOLABEL:
-                return
-        for halfface in cell.halffaces:
-            if halfface.face.type != 'face':
-                continue
-            symbol = halfface.face.id
-            faceno = self.faces.index(symbol)
-            cell_vertices = [vectordata.toindex(he.verts[0].point) for he in halfface.halfedges]
-            cell_vertices.append(faceno)
-            yield cell_vertices
-            
-    @staticmethod
-    def gl_black_polygons(cell, inormals, vectordata):
-        if DEBUG:
-            if DEBUG_NOBEVEL:
-                return
-        for halfface in cell.halffaces:
-            if halfface.face.type == 'face':
-                continue
-            vertices = [he.verts[0] for he in halfface.halfedges]
-            if halfface.face.type == 'cut':
-                inormal = inormals[(cell.indices, halfface.face.id)]
-                vnit = ((vectordata.toindex(v.point), inormal) for v in vertices)
-                yield list(chain(*vnit))
-            elif halfface.face.type == 'bevel':
-                vnit = ((vectordata.toindex(v.point), inormals[(cell.indices, v.id)])
-                            for v in vertices)
-                yield list(chain(*vnit))
-                
+    def gl_block_data(self, polys, vectordata):
+        polys_faces = polys.faces
+        def gl_block_data_cell(cell):
+            def is_visible(vert):
+                for hv in vert.halfverts:
+                    he = hv.halfedge
+                    hf = he.halfface
+                    if hv is he.halfverts[1] and hf.hdim is cell and not hf.face.type.endswith('_removed'):
+                        return True
+                return False
+            def gen_cedges(cell_halffaces):
+                edgesdone = []
+                for hf in cell_halffaces:
+                    for he1 in hf.halfedges:
+                        if he1.edge not in edgesdone:
+                            he3 = he1.other
+                            hf3_face = he3.halfface.face
+                            edgesdone.append(he1.edge)
+                            if not (hf.face.type.endswith('_removed') and hf3_face.type.endswith('_removed')):
+                                v1_point = vectordata.toindex(he1.verts[1].point)
+                                v3_point = vectordata.toindex(he3.verts[1].point)
+                                yield polys_faces.index(hf.face), polys_faces.index(hf3_face), v1_point, v3_point
+            def gen_vhalfface(halfface, vert):
+                for hv in vert.halfverts:
+                    he = hv.halfedge
+                    hf = he.halfface
+                    if hv is he.halfverts[1] and hf is halfface:
+                        break
+                else:
+                    assert False
+                while True:
+                    assert hv.vert is vert
+                    yield polys_faces.index(hf.face)
+                    he = hv.halfedge.other
+                    hv = he.halfverts[0].other
+                    assert hv.vert is vert
+                    hf = he.halfface
+                    if hf is halfface:
+                        break
+            def gen_cverts(cell_halffaces):
+                vertsdone = []
+                for hf in cell_halffaces:
+                    for he1 in hf.halfedges:
+                        he3 = he1.other
+                        hf3 = he3.halfface
+                        v1 = he1.verts[1]
+                        v3 = he3.verts[1]
+                        if v1 not in vertsdone:
+                            vertsdone.append(v1)
+                            if is_visible(v1):
+                                vertdata = [f for f in gen_vhalfface(hf, v1)]
+                                yield vectordata.toindex(v1.point), vertdata
+                        if v3 not in vertsdone:
+                            vertsdone.append(v3)
+                            if is_visible(v3):
+                                vertdata = [f for f in gen_vhalfface(hf3, v3)]
+                                yield vectordata.toindex(v3.point), vertdata
+            def gen_cfaces(cell_halffaces):
+                c_faces_label = []
+                c_faces_black = []
+                for hf in cell_halffaces:
+                    verts = [vectordata.toindex(v.point) for v in hf.verts]
+                    if hf.face.type == 'face':
+                        c_faces_label.append((polys_faces.index(hf.face), self.faces.index(hf.face.id), verts))
+                    elif hf.face.type == 'cut':
+                        c_faces_black.append((polys_faces.index(hf.face), verts))
+                    else:
+                        assert hf.face.type.endswith('_removed')
+                return c_faces_label, c_faces_black
+            c_faces_label, c_faces_black = list(gen_cfaces(cell.halffaces))
+            c_edges = list(gen_cedges(cell.halffaces))
+            c_verts = list(gen_cverts(cell.halffaces))
+            return c_faces_label, c_faces_black, c_edges, c_verts
+        return [gl_block_data_cell(cell) for cell in polys.cells]
+        
     def gl_pick_polygons(self, cells, vectordata):
+        black_faces = []
         for cellidx, cell in enumerate(cells):
+            cell_facesdata = []
             for halfface in cell.halffaces:
-                if halfface.face.type != 'face':
-                    continue
-                symbol = halfface.face.id
-                face = self.faces.index(symbol)
-                visible_face_indices = [i for i, he in enumerate(halfface.halfedges)
+                if halfface.face.type == 'face':
+                    symbol = halfface.face.id
+                    face = self.faces.index(symbol)
+                    visible_face_indices = [i for i, he in enumerate(halfface.halfedges)
                                           if he.other.halfface.face.type == 'face']
+                else:
+                    if halfface.face in black_faces:
+                        continue
+                    black_faces.append(halfface.face)
+                    face = halfface.face.id
                 edges_iverts = [vectordata.toindex(he.verts[0].point) for he in halfface.halfedges]
+                if halfface.face.type != 'face':
+                    cell_facesdata.append([face, None, edges_iverts])
+                    continue
                 if len(visible_face_indices) == 0:
                     picktype = 0
                 elif len(visible_face_indices) == 1:
@@ -352,9 +385,9 @@ class ModelFactory:
                         picktype = -1
                 else:
                     picktype = -1
-                yield [cellidx, [face, picktype], edges_iverts]
+                cell_facesdata.append([face, picktype, edges_iverts])
+            yield cell_facesdata
                 
-        
 
 class Dedup:
     def __init__(self):
@@ -424,24 +457,26 @@ class Dedup:
             return value, False, vtype
             
     def dedup(self, value):
-        if isinstance(value, dict):
+        if type(value) is dict:
             value = {(sys.intern(k) if type(k) is str else k):v for k,v in value.items()}
             dedup, vtype = self._recursion(value, value.items())
             return self._replace(value, dedup, 'd' + vtype)
-        elif isinstance(value, list):
+        elif type(value) is list:
             dedup, vtype = self._recursion(value, enumerate(value))
             return self._replace(tuple(value), dedup, 'a' + vtype)
-        elif isinstance(value, tuple):
+        elif type(value) is tuple:
             value = list(value)
             dedup, vtype = self._recursion(value, enumerate(value))
             return self._replace(tuple(value), dedup, 'a' + vtype)
-        elif isinstance(value, float):
+        elif type(value) is float:
             return self._float(value)
-        elif isinstance(value, int):
+        elif type(value) is int:
             return value, True, 'i'
-        elif isinstance(value, str):
+        elif type(value) is str:
             return sys.intern(value), True, 's'
-        elif isinstance(value, type(None)):
+        elif type(value) is bool:
+            return value, True, 'b'
+        elif value is None:
             return value, True, 'n'
         else:
             assert False, type(value)
@@ -464,43 +499,38 @@ def pool_functions(parallel, reproducible):
         return map
         
 #XXX: speedup parallel builds, longer jobs first
-fileorder = 'b111 b101 b000 p111111 b010 p101111 b100 b110 p010000 p000000 b011 t1111 b001 t0000 t1011 t0100'.split() 
+fileorder = 'b111 d000000 b101 b000 b010 p111111 p101111 b100 t1111 b110 p010000 b011 p000000 t0000 b001 t1011 t0100'.split()
 
-def get_datafilename(fileclass, sizes, fast):
-    if fast:
-        return fileclass + ''.join(str(s-1) for s in sizes)
+def get_datafilename(Factory, sizes):
+    if DEBUG_MODELFAST:
+        return 'f{:02}{}{}'.format(modeldefs.index(Factory)+1, Factory.fileclass, ''.join(str(s-1) for s in sizes))
     else:
-        sizes = fileclass + ''.join(str((s-1)%2) for s in sizes)
+        sizes = Factory.fileclass + ''.join(str((s-1)%2) for s in sizes)
         try:
             priority = fileorder.index(sizes) + 1
         except ValueError:
             priority = 99
-        return '{:02}'.format(priority) + sizes
+        return 'd{:02}'.format(priority) + sizes
         
-def enum_modelfiles(fast, maxsize, Factory):
-    if isinstance(maxsize, tuple):
-        minsize, maxsize = maxsize
-    else:
-        minsize = 1
-    maxsize += 1
+def enum_modelfiles(Factory):
     def tuples(maxlen, part=()):
         if maxlen == len(part):
             yield part
         else:
-            for i in range(minsize, maxsize):
+            for i in range(minsize, maxsize+1):
                 yield from tuples(maxlen, part+(i,))
     for size in tuples(len(Factory.sizenames)):
         norm_size, sizes = Factory.norm_sizes(size)
         if sizes is None:
             continue
-        filename = get_datafilename(Factory.fileclass, sizes, fast)
+        filename = get_datafilename(Factory, sizes)
         yield filename, sizes, size, norm_size
             
-def pool_enum_modelfiles(dirname, testfunc, fast, maxsize):
+def pool_enum_modelfiles(dirname, testfunc):
     ignored = []
     modelfiles = []
     for Factory in modeldefs:
-        for filename, *unused in enum_modelfiles(fast, maxsize, Factory):
+        for filename, *unused in enum_modelfiles(Factory):
             filename = os.path.join(dirname, filename)
             if filename in modelfiles or filename in ignored:
                 continue
@@ -513,14 +543,14 @@ def pool_enum_modelfiles(dirname, testfunc, fast, maxsize):
             print('skipping', filename)
     return sorted(modelfiles)
     
-def pool_create_modelfiledata(path, fast, maxsize):
+def pool_create_modelfiledata(path):
     filename = os.path.basename(path)
     savedata = {}
-    vectordata = VectorData(fast)
+    vectordata = VectorData()
     for Factory in modeldefs:
         sizes_list = []
         factory = None
-        for _filename, sizes, *unused in enum_modelfiles(fast, maxsize, Factory):
+        for _filename, sizes, *unused in enum_modelfiles(Factory):
             if _filename != filename:
                 continue
             if sizes in sizes_list:
@@ -529,8 +559,7 @@ def pool_create_modelfiledata(path, fast, maxsize):
             if factory is None:
                 factory = ModelFactory(Factory)
             savedata.setdefault(factory.type, {})[sizes] = factory.get_data(sizes, vectordata)
-    vectors = array('f', chain.from_iterable(vectordata.vectors))
-    return savedata, vectors, vectordata.cnts, vectordata.dups
+    return savedata, vectordata.vectors, vectordata.cnts, vectordata.dups
     
 def format_seconds(seconds):
     if seconds > 60:
@@ -538,41 +567,110 @@ def format_seconds(seconds):
     else:
         return '{:.2f}s'.format(seconds)
     
-def pool_create_modelfile(filename, fast, pickle_protocol, maxsize):
+def pool_check_compare_modelfiledata(check_savedata, check_vectors, savedata, vectors):
+    if savedata == check_savedata and vectors == check_vectors:
+        return 'check: pass full'
+        
+    for mtype, check_value_mtype in list(check_savedata.items()):
+        if mtype not in savedata:
+            del check_savedata[mtype]
+        else:
+            value_mtype = savedata[mtype]
+            for sizes, check_value_sizes in list(check_value_mtype.items()):
+                if sizes not in value_mtype:
+                    del check_value_mtype[sizes]
+                else:
+                    value_sizes = value_mtype[sizes]
+                    for attr in 'block_polygons', 'pick_polygons':  # older versions may have this attrs
+                        with suppress(KeyError):
+                            del check_value_sizes[attr]
+                    for attr in 'facesdata', 'blocksdata', 'normals':
+                        with suppress(KeyError):
+                            del check_value_sizes[attr]
+                        del value_sizes[attr]
+    for mtype, value_mtype in list(savedata.items()):
+        if mtype not in check_savedata:
+            del savedata[mtype]
+        else:
+            for sizes in list(value_mtype.keys()):
+                if sizes not in check_savedata[mtype]:
+                    del value_mtype[sizes]
+    if savedata == check_savedata:
+        return 'check: pass weak'
+    return 'check: data different'
+    
+def write_diff(filename, check_savedata, savedata):
+    from difflib import unified_diff
+    from pprint import pformat
+    columns = int(os.environ.get('COLUMNS') or '80') - 1
+    diff = unified_diff(pformat(check_savedata, width=columns).splitlines(),
+                        pformat(savedata, width=columns).splitlines(),
+                        fromfile=filename, tofile='<generated by {}>'.format(__file__),
+                        n=3, lineterm='')
+    filename += '.diff'
+    isempty = True
+    with open(filename, 'wt') as file:
+        for line in diff:
+            print(line, file=file)
+            isempty = False
+    if isempty and os.path.exists(filename):
+        os.remove(filename)
+            
+def pool_create_modelfile(filename, pickle_protocol, check):
     import time
     seconds = time.process_time()
     
-    savedata, vectors, cnt_vectors, dup_vectors = pool_create_modelfiledata(filename, fast, maxsize)
+    if check:
+        import pickle
+        import pybiklib.model
+        try:
+            with open(filename, 'rb') as datafile:
+                check_savedata = pickle.load(datafile)
+                check_vectors = pybiklib.model.Model.read_vectors(datafile)
+        except FileNotFoundError:
+            check_savedata = {}
+            check_vectors = []
+        check = (check_savedata, check_vectors)
+    else:
+        check = None
+        
+    savedata, vectors, cnt_vectors, dup_vectors = pool_create_modelfiledata(filename)
     vals = dups = ''
-    if not fast:
+    if not DEBUG_MODELFAST:
         dedup = Dedup()
         savedata = dedup.dedup(savedata)[0]
         if DEBUG_MSG:
             vals = '\n  vals: %6s  vec: %6s' % (dedup.cnt_values, cnt_vectors)
             dups = '\n  dups: %6s       %6s' % (dedup.cnt_dups, dup_vectors)
     
-    savedata = dumps(savedata, pickle_protocol)
-    datasize = ['{:.1f} kb'.format(len(savedata) / 1000), '---']
+    bsavedata = dumps(savedata, pickle_protocol)
+    datasize = ['{:.1f} kb'.format(len(bsavedata) / 1000), '---']
+    vectors = array('f', chain.from_iterable(vectors))
     vectorssize = '{:.1f} kb'.format(len(vectors) * vectors.itemsize / 1000)
-    if not fast:
-        savedata = pickletools.optimize(savedata)
-        datasize[1] = '{:.1f} kb'.format(len(savedata) / 1000)
+    if not DEBUG_MODELFAST:
+        bsavedata = pickletools.optimize(bsavedata)
+        datasize[1] = '{:.1f} kb'.format(len(bsavedata) / 1000)
     seconds = time.process_time() - seconds
     
-    vlen = array('I', [len(vectors)])
-    if sys.byteorder != filebyteorder:
-        vlen.byteswap()
-        vectors.byteswap()
-    with open(filename, 'wb') as datafile:
-        datafile.write(savedata)
-        vlen.tofile(datafile)
-        vectors.tofile(datafile)
-    return filename, seconds, 'generated {}, ({}, {}), {}, {} vectors {}{}{}'.format(
-                    filename,
+    if check is None:
+        vlen = array('I', [len(vectors)])
+        if sys.byteorder != filebyteorder:
+            vlen.byteswap()
+            vectors.byteswap()
+        with open(filename, 'wb') as datafile:
+            datafile.write(bsavedata)
+            vlen.tofile(datafile)
+            vectors.tofile(datafile)
+        message = 'generated'
+    else:
+        message = pool_check_compare_modelfiledata(check_savedata, check_vectors, savedata, vectors)
+        write_diff(filename, check_savedata, savedata)
+    return filename, seconds, '{} {:{}} ({:>9}, {:>9}), {:>9}, {:7} vectors {:>8}{}{}'.format(
+                    message, filename, len(os.path.dirname(filename))+11,
                     datasize[0], datasize[1], format_seconds(seconds), len(vectors) / 3, vectorssize,
                     vals, dups)
     
-def pool_create_indexdata(fast, maxsize):
+def pool_create_indexdata():
     savedata_type = {}
     savedata_types = []
     savedata_size = {}
@@ -585,7 +683,7 @@ def pool_create_indexdata(fast, maxsize):
     facekeys = []
     for Factory in modeldefs:
         factory = ModelFactory(Factory)
-        savedata_type[factory.type] = {attr: getattr(factory, attr) for attr in factory.mtype_attributes}
+        savedata_type[factory.type] = factory.getattrs()
         savedata_types.append(factory.type)
         savedata_size_type = {}
         savedata_sizes_type = {}
@@ -595,7 +693,7 @@ def pool_create_indexdata(fast, maxsize):
             if facekey not in facekeys:
                 facekeys.append(facekey)
                 savedata_facenames.append((facekey, facename))
-        for filename, sizes, size, norm_size in enum_modelfiles(fast, maxsize, Factory):
+        for filename, sizes, size, norm_size in enum_modelfiles(Factory):
             savedata_size_type[size] = norm_size
             if norm_size in savedata_sizes_type:
                 assert [sizes, filename] == savedata_sizes_type[norm_size]
@@ -607,29 +705,65 @@ def pool_create_indexdata(fast, maxsize):
         savedata_type[factory.type]['defaultsize'] = defaultsize
     return savedata
     
-def pool_create_indexfile(filename, fast, pickle_protocol, maxsize):
+def pool_check_compare_indexdata(check, savedata):
+    if check == savedata:
+        return 'check: pass full'
+        
+    def remove_mtype(data, mtype):
+        del data['normsize'][mtype]
+        del data['sizes'][mtype]
+        del data['type'][mtype]
+    for mtype in check['types']:
+        if mtype not in savedata['types']:
+            remove_mtype(check, mtype)
+    for mtype in savedata['types']:
+        if mtype not in check['types']:
+            remove_mtype(savedata, mtype)
+    for attr in 'normsize','sizes',:
+        for mtype, check_value in check[attr].items():
+            savedata_value = savedata[attr][mtype]
+            for key in list(check_value.keys()):
+                if key not in savedata_value:
+                    del check_value[key]
+            
+    if check == savedata:
+        return 'check: pass weak'
+    return 'check: data different'
+        
+def pool_create_indexfile(filename, pickle_protocol, check):
     import time
     seconds = time.process_time()
     
-    savedata = pool_create_indexdata(fast, maxsize)
+    if check:
+        import pybiklib.model
+        pybiklib.model.Model.load_index()
+        check = pybiklib.model.Model.cache_index
+    else:
+        check = None
+    savedata = pool_create_indexdata()
     vals = dups = ''
-    if not fast:
+    if not DEBUG_MODELFAST:
         dedup = Dedup()
         savedata = dedup.dedup(savedata)[0]
         if DEBUG_MSG:
             vals = '\n  vals: %6s' % dedup.cnt_values
             dups = '\n  dups: %6s' % dedup.cnt_dups
-    savedata = dumps(savedata, pickle_protocol)
-    datasize = ['{:.1f} kb'.format(len(savedata) / 1000), '---']
-    if not fast:
-        savedata = pickletools.optimize(savedata)
-        datasize[1] = '{:.1f} kb'.format(len(savedata) / 1000)
+    bsavedata = dumps(savedata, pickle_protocol)
+    datasize = ['{:.1f} kb'.format(len(bsavedata) / 1000), '---']
+    if not DEBUG_MODELFAST:
+        bsavedata = pickletools.optimize(bsavedata)
+        datasize[1] = '{:.1f} kb'.format(len(bsavedata) / 1000)
     seconds = time.process_time() - seconds
     
-    with open(filename, 'wb') as datafile:
-        datafile.write(savedata)
-    return filename, seconds, 'generated {}, ({}, {}), {}{}{}'.format(
-                    filename,
+    if check is None:
+        with open(filename, 'wb') as datafile:
+            datafile.write(bsavedata)
+        message = 'generated'
+    else:
+        message = pool_check_compare_indexdata(check, savedata)
+        write_diff(filename, check, savedata)
+    return filename, seconds, '{} {:{}} ({:>9}, {:>9}), {:>9}{}{}'.format(
+                    message, filename, len(os.path.dirname(filename))+11,
                     datasize[0], datasize[1], format_seconds(seconds),
                     vals, dups)
     
@@ -638,19 +772,20 @@ def pool_run(args):
     return func(*args)
     
 def get_indexfilename(dirname):
-    return os.path.join(dirname, 'index')
+    return os.path.join(dirname, 'f00index' if DEBUG_MODELFAST else 'd00index')
     
-def create_modeldata(dirname, testfunc=None, parallel=1, fast=False, pickle_protocol=-1, maxsize=10, reproducible=False):
+def create_modeldata(dirname, testfunc=None, parallel=1, pickle_protocol=-1,
+                              reproducible=False, check=False):
     # prepare jobs
     if testfunc is None:
         testfunc = lambda arg: True
     if DEBUG_INDEXONLY:
         modelfiles = []
     else:
-        modelfiles = pool_enum_modelfiles(dirname, testfunc, fast, maxsize)
-    jobs = [(pool_create_modelfile, m, fast, pickle_protocol, maxsize) for m in modelfiles]
+        modelfiles = pool_enum_modelfiles(dirname, testfunc)
+    jobs = [(pool_create_modelfile, m, pickle_protocol, check) for m in modelfiles]
     indexfilename = get_indexfilename(dirname)
-    jobs.append((pool_create_indexfile, indexfilename, fast, pickle_protocol, maxsize))
+    jobs.append((pool_create_indexfile, indexfilename, pickle_protocol, check))
     # run jobs
     if parallel is True:
         parallel = cpu_count()
@@ -664,7 +799,7 @@ def create_modeldata(dirname, testfunc=None, parallel=1, fast=False, pickle_prot
         result.append((filename, seconds))
         print(lines)
         sys.stdout.flush()
-    if DEBUG_MSG and not fast:
+    if DEBUG_MSG and not DEBUG_MODELFAST:
         result.sort(key=lambda fs: fs[1], reverse=True)
         if result != sorted(result, key=lambda fs: fs[0]):
             print('warning: reorder jobs to optimize parallel build')
@@ -673,6 +808,10 @@ def create_modeldata(dirname, testfunc=None, parallel=1, fast=False, pickle_prot
     
     
 if __name__ == '__main__':
-    create_modeldata('data/models', maxsize=(1,6))
+    minsize = 1
+    maxsize = 5
+    create_modeldata('data/models',
+                     check=True,
+                    )
     
 
