@@ -1,6 +1,6 @@
 #-*- coding:utf-8 -*-
 
-#  Copyright © 2009, 2011-2015  B. Clausius <barcc@gmx.de>
+#  Copyright © 2009, 2011-2017  B. Clausius <barcc@gmx.de>
 #
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -16,20 +16,25 @@
 #  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
-
-import os, sys
+import os
 
 N_ = lambda s: s
+try:
+    _
+except NameError:
+    _ = N_
 
 AUTHOR = 'B. Clausius'
 CONTACT_EMAIL = 'barcc@gmx.de'
 CONTACT_WEB = 'https://answers.launchpad.net/pybik'
-COPYRIGHT = 'Copyright © 2009-2015,  B. Clausius'
-LICENSE_NAME = 'GPL-3+'
+COPYRIGHT = 'Copyright © 2009-2017,  B. Clausius'
+LICENSE_NAME = 'GPL-3.0+'
 PACKAGE = 'pybik'
-# Name of the application, probably should not be translated.
+PACKAGE_FULL = 'pybik.barcc'
+#TRANSLATORS: Name of the application, probably should not be translated.
 APPNAME = N_('Pybik')
-VERSION = '2.1'
+VERSION = '3.0'
+VERSION_SHORT = VERSION.split('~')[0]
 WEBSITE = 'https://launchpad.net/pybik/'
 DOWNLOADS = 'https://launchpad.net/pybik/+download'
 BUG_DATABASE = 'https://bugs.launchpad.net/pybik'
@@ -40,43 +45,33 @@ CONTACT_FILEBUG = BUG_DATABASE + '/+filebug'
 data_dir = os.path.join(os.path.dirname(os.path.dirname(__file__)), 'data')
 appdata_dir = data_dir
 # The following line is replaced by "setup.py sdist"
-RELEASE_DATE = '2015-07-29'
+RELEASE_DATE = '2017-10-22'
 
 LOCALE_DIR   = os.path.join(data_dir, 'locale')
-QT_LOCALE_DIR = os.path.join(sys.prefix, 'share', 'qt5', 'translations')
 APPICON_FILE = os.path.join(appdata_dir, 'icons', 'pybik-64.png')
 APPICON192_FILE = os.path.join(appdata_dir, 'icons', 'pybik-192.png')
 LICENSE_FILE = os.path.join(appdata_dir, 'GPL-3')
 MODELS_DIR   = os.path.join(appdata_dir, 'models')
 PLUGINS_DIR  = os.path.join(appdata_dir, 'plugins')
 UI_DIR       = os.path.join(appdata_dir, 'ui')
-TESTDATA_DIR = os.path.join(appdata_dir, 'tests')
-SHADER_DIR = os.path.join(appdata_dir, 'shaders')
+SHADER_DIR   = os.path.join(appdata_dir, 'shaders')
 
-def get_testdatafiles():
-    return [f for f in os.listdir(TESTDATA_DIR) if not f.endswith('~')]
+def get_config_home(*args):
+    return os.path.join(os.environ.get('XDG_CONFIG_HOME', '') or os.path.expanduser("~/.config"), *args)
+def get_data_home(*args):
+    return os.path.join(os.environ.get('XDG_DATA_HOME', '') or os.path.expanduser("~/.local/share"), *args)
     
-def get_config_home():
-    return os.environ.get('XDG_CONFIG_HOME', '') or os.path.expanduser("~/.config")
-def get_data_home():
-    return os.environ.get('XDG_DATA_HOME', '') or os.path.expanduser("~/.local/share")
-    
-USER_CONFIG_DIR    = os.path.join(get_config_home(), PACKAGE)
-USER_DATA_DIR      = os.path.join(get_data_home(), PACKAGE)
-USER_SETTINGS_FILE = os.path.join(USER_CONFIG_DIR, 'settings.conf')
-USER_PLUGINS_DIR   = os.path.join(USER_DATA_DIR, 'plugins')
-USER_GAMES_FILE    = os.path.join(USER_DATA_DIR, 'games')
-
+#TRANSLATORS: short description
 SHORT_DESCRIPTION = N_("Rubik's cube game")
 LONG_DESCRIPTION = (
-    # The next 7 lines belong together and form the long description
+    #TRANSLATORS: The next 7 lines belong together and form the long description
     N_('Pybik is a 3D puzzle game about the cube invented by Ernő Rubik.'),
-    N_('* Different 3D puzzles - up to 10x10x10:'),
-    N_('  cubes, towers, bricks, tetrahedra and prisms'),
-    N_('* Solvers for some puzzles'),
-    N_('* Pretty patterns'),
-    N_('* Editor for move sequences'),
-    N_('* Changeable colors and images'),
+    '',
+    '* '+N_('Various 3D puzzles, such as cubes, towers, prisms, tetrahedra, octahedra, dodecahedra and many more'),
+    '* '+N_('Solvers for some puzzles'),
+    '* '+N_('Pretty patterns'),
+    '* '+N_('Editor for move sequences'),
+    '* '+N_('Custom colors and images on the surface of the puzzles'),
 )
 
 LICENSE_INFO = N_(
@@ -90,8 +85,8 @@ LICENSE_INFO = N_(
             'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the '
             'GNU General Public License for more details.')
 LICENSE_FURTHER = N_(
-# Text between "<" and ">" is expanded to a link by the program and should not be modified.
-# Text between "<LINK|>" and "<|>" is the translatable text for the link.
+#TRANSLATORS: Text between "<" and ">" is expanded to a link by the program and should not be modified.
+#             Text between "<LINK|>" and "<|>" is the translatable text for the link.
             'Read the full text of the <text:FULL_LICENSE_TEXT|>GNU General Public License'
             '<|> or see <http://www.gnu.org/licenses/>.')
 LICENSE_NOT_FOUND = N_(
@@ -100,10 +95,8 @@ LICENSE_NOT_FOUND = N_(
 
 def get_filebug_text():
     return _(
-    # "Wishlist" is used on Launchpad for Importance, so this word should probably not be translated
     'If you find any bugs in Pybik or have a suggestion for an improvement then please '
-    'submit a <{CONTACT_FILEBUG}|>bug report<|>. '
-    'In the latter case you can mark the bug report as "Wishlist".'
+    'submit a <{CONTACT_FILEBUG}|>bug report<|>.'
     ).format(CONTACT_FILEBUG=CONTACT_FILEBUG)
 
 TRANSLATION_TEXT = N_(
@@ -118,5 +111,6 @@ TRANSLATION_TEXT = N_(
 REPOSITORY_SHORT = 'lp:pybik'
 REPOSITORY_URL = 'https://code.launchpad.net/~barcc/pybik/trunk'
 REPOSITORY_BROWSE = 'https://bazaar.launchpad.net/~barcc/pybik/trunk/files'
+REPOSITORY_DOWNLOAD = 'http://bazaar.launchpad.net/~barcc/pybik/trunk/download'
 
 
