@@ -1,6 +1,6 @@
 #-*- coding:utf-8 -*-
 
-#  Copyright © 2009-2015  B. Clausius <barcc@gmx.de>
+#  Copyright © 2009-2015, 2017  B. Clausius <barcc@gmx.de>
 #
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 import random
 from copy import deepcopy
 
-from .debug import debug, DEBUG_MSG, DEBUG_RAND, DEBUG_ROTATE
+from .debug import DEBUG_MSG, DEBUG_RAND, DEBUG_ROTATE
 from .moves import MoveQueuePacked, MoveData
 
 
@@ -46,7 +46,8 @@ class CubeState:
         
     @property
     def rotations(self):
-        return [blockn[1] for blockn in self._blocksn]
+        return [(idx, self.model.rotation_indices[rot], rot)
+                    for pos, (idx, rot) in enumerate(self._blocksr)]
         
     def set_solved(self):
         # '' symbolic rotation for identity
@@ -88,6 +89,12 @@ class CubeState:
                 return False  # Cubie at wrong place
         return True
         
+    def is_identity(self):
+        for index, (pos, rot) in enumerate(self._blocksn):
+            if index != pos or rot != '':
+                return False
+        return True
+        
     def identify_rotation_blocks(self, maxis, mslice):
         if mslice == -1:
             for i, blockn in enumerate(self._blocksn):
@@ -158,38 +165,25 @@ class CubeState:
         
     def random(self, count=-1):
         sizes = self.model.sizes
-        count_moves = sum(sizes)*2
+        all_moves = [(a,s,d) for a,_sz in enumerate(sizes) for s in range(_sz) for d in (False,True)]
         randrange = self.rand.randrange
         if count < 0:
-            count = 10 * count_moves + randrange(count_moves)
+            count = 10 * len(all_moves) + randrange(len(all_moves))
         if count:
             # The solved cube should be randomly rotated (lp: 1411999)
             for sym in self.rand.choice(self.model.rotation_symbols):
                 self.rotate_slice(self.model.sym_to_move(sym))
         packed = MoveQueuePacked()
-        #FIXME: workaround: for random Prism5* the last slice should not be used
-        prism5workaround = count and self.model.type.startswith('Prism5')
+        all_moves = [(a,s,d) for a,s,d in all_moves if self.model.is_user_slice(a, s)]
         for i in range(count):
-            candidates = list(range(count_moves))
+            candidates = list(range(len(all_moves)))
             while candidates:
-                mslice = candidates.pop(randrange(len(candidates)))
-                mdir = bool(mslice % 2)
-                mslice //= 2
-                for maxis, size in enumerate(sizes):
-                    if mslice < size:
-                        break
-                    mslice -= size
-                else:
-                    assert False
-                if prism5workaround and maxis != 1:
-                    if mslice == size-1:
-                        continue
-                move = packed.pack(MoveData(maxis, mslice, mdir), self.model)
-                if packed.length() > i:
-                    if (prism5workaround and maxis != 1) or not bool(move.get_full_moves(self.model)):
-                        self._rotate_slice(maxis, mslice, mdir)
-                        break
-                packed.pack(MoveData(maxis, mslice, not mdir), self.model)
+                maxis, mslice, mdir = all_moves[candidates.pop(randrange(len(candidates)))]
+                packed.push_unpacked(MoveData(maxis, mslice, mdir), self.model)
+                if packed.length_no_full(self.model) > i:
+                    self._rotate_slice(maxis, mslice, mdir)
+                    break
+                packed.push_unpacked(MoveData(maxis, mslice, not mdir), self.model)
             else:
                 break
         if DEBUG_MSG:
