@@ -1,7 +1,7 @@
 #!/usr/bin/python3
 # -*- coding: utf-8 -*-
 
-#  Copyright © 2012-2015  B. Clausius <barcc@gmx.de>
+#  Copyright © 2012-2017  B. Clausius <barcc@gmx.de>
 #
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -21,10 +21,13 @@ import sys, os
 import re
 import pickle
 from array import array
+from enum import Enum
 
-from .debug import DEBUG_DRAW, debug
+from .debug import (debug, DEBUG_DRAW, DEBUG_DRAWNORMALS, DEBUG_DRAWAXES,
+                           DEBUG_NOPICK, DEBUG_NOBEVEL, DEBUG_NOLABEL, DEBUG_NOBLACK,
+                           DEBUG_NOCACHE, DEBUG_MODELFAST, DEBUG_CACHEUPDATE)
 from .config import MODELS_DIR
-from .modelcommon import filebyteorder, epsilon, get_texcoords_range, Vector, epsdigits
+from .utils import filebyteorder, epsilon, get_texcoords_range, Vector
 
 try:
     _
@@ -38,6 +41,7 @@ def get_texcoords(vector, normal, minx, maxx, miny, maxy):
     
         
 class EmptyModel:
+    type = ''
     symbols = ''
     symbolsI = ''
     faces = ''
@@ -55,13 +59,15 @@ class EmptyModel:
         return 'no puzzle'
         
     def gl_vertex_data(self, *unused_args):
-        return (0, b'', (0, 0, 0, 0, 0, 0), ([], 0, 0), []), [()]
+        return (0, b'', (0, 0, 0, 0, 0, 0), ([], 0, 0, 0), []), [()]
         
         
 class Model:
+    bevel_width = 0.01 if DEBUG_NOBEVEL else 0.1
+    
     cache_index = {}
     cache_data = {}
-    cache_vectors = []
+    cache_vectors = {} if DEBUG_CACHEUPDATE else []
     
     def __init__(self, mtype, size):
         items = self.cache_index['type'][mtype].items()
@@ -78,6 +84,18 @@ class Model:
         self.cell_count = len(self.cell_indices)
         self.rotation_indices = {s:i for i, s in enumerate(self.rotation_symbols)}
         self._inplace_rotations = None
+        if self.slicesmode == 'all':
+            self.ignored_slice = [None for sz in self.sizes]
+        elif self.slicesmode == 'mid':
+            self.ignored_slice = [(sz-1)//2 for sz in self.sizes]
+        elif self.slicesmode == 'last1':
+            self.ignored_slice = [(None if a == 1 else sz-1) for a, sz in enumerate(self.sizes)]
+        else:
+            assert False, self.slicesmode
+        
+        #XXX: self.beveled_fvertdata generated here from a nested list, but
+        #     the idices come from the modeldata files
+        self.beveled_fvertdata = self.gl_bevel_faces()
         
     @property
     def inplace_rotations(self):
@@ -93,24 +111,30 @@ class Model:
                 self._inplace_rotations.append(sorted(inplace_rotations, key=len)[:2])
         return self._inplace_rotations
         
+    def is_user_slice(self, iaxis, mslice):
+        return self.ignored_slice[iaxis] is None or self.ignored_slice[iaxis] != mslice
+        
     def __str__(self):
-        return self.mformat.format(*self.sizes)
+        return self.mformat.format(*self.size)
         
     @classmethod
     def load_index(cls):
-        indexfilename = os.path.join(MODELS_DIR, 'index')
+        if DEBUG_NOCACHE:
+            try:
+                from buildlib import modeldata
+            except ImportError:
+                pass
+            else:
+                debug('generating model index')
+                cls.cache_index = modeldata.pool_create_indexdata()
+                return
+        indexfilename = os.path.join(MODELS_DIR, 'f00index' if DEBUG_MODELFAST else 'd00index')
+        debug('loading model index:', indexfilename)
         with open(indexfilename, 'rb') as indexfile:
             cls.cache_index = pickle.load(indexfile)
             
     @staticmethod
     def read_vectors(datafile):
-        def isplit3(vals):
-            it = iter(vals)
-            for x in it:
-                try:
-                    yield Vector((x, next(it), next(it)))
-                except StopIteration:
-                    raise ValueError('len must be a multiple of 3')
         vlen = array('I')
         vlen.fromfile(datafile, 1)
         if sys.byteorder != filebyteorder:
@@ -120,21 +144,50 @@ class Model:
         vectors.fromfile(datafile, vlen)
         if sys.byteorder != filebyteorder:
             vectors.byteswap()
-        return list(isplit3(vectors))
+        return vectors
+        
+    @staticmethod
+    def isplit3_vectors(vals):
+        it = iter(vals)
+        for x in it:
+            try:
+                yield Vector((x, next(it), next(it)))
+            except StopIteration:
+                raise ValueError('len must be a multiple of 3')
+                
+    @classmethod
+    def _load_data(self, filename):
+        if DEBUG_NOCACHE:
+            try:
+                from buildlib import modeldata
+            except ImportError:
+                pass
+            else:
+                print('generating model data:', filename)
+                _data, _vectors, *unused = modeldata.pool_create_modelfiledata(filename)
+                _vectors = [Vector(v) for v in _vectors]
+                return _data, _vectors
+        filename = os.path.join(MODELS_DIR, filename)
+        debug('loading model data:', filename)
+        with open(filename, 'rb') as datafile:
+            data = pickle.load(datafile)
+            vectors = list(self.isplit3_vectors(self.read_vectors(datafile)))
+        return data, vectors
         
     def load_data(self, filename):
+        debug('requesting model data:', filename, self.type, self.sizes)
         try:
-            data = self.cache_data[self.type][self.sizes]
+            data = (self.cache_data[filename] if DEBUG_CACHEUPDATE else self.cache_data)[self.type][self.sizes]
         except KeyError:
-            filename = os.path.join(MODELS_DIR, filename)
-            debug('loading model data:', filename)
-            with open(filename, 'rb') as datafile:
-                data = pickle.load(datafile)
-                vectors = self.read_vectors(datafile)
-            self.__class__.cache_data = data
-            self.__class__.cache_vectors = vectors
+            data, vectors = self._load_data(filename)
+            if DEBUG_CACHEUPDATE:
+                self.cache_data[filename] = data
+                self.cache_vectors[filename] = vectors
+            else:
+                self.__class__.cache_data = data
+                self.__class__.cache_vectors = vectors
             data = data[self.type][self.sizes]
-        self.vectors = self.cache_vectors
+        self.vectors = self.cache_vectors[filename] if DEBUG_CACHEUPDATE else self.cache_vectors
         for key, value in data.items():
             setattr(self, key, value)
             
@@ -181,10 +234,6 @@ class Model:
                     raise ValueError('invalid symbol:', sym)
             return new_sym
             
-    def rotations_symbolic_to_index(self, rotations):
-        for sym in rotations:
-            yield self.rotation_indices[sym]
-        
     def blocksym_to_blockpos(self, blocksym):
         for i, faces in enumerate(self.cells_visible_faces):
             #FIXME: different blocks can have same visible faces (eg. edges for 4-Cubes)
@@ -209,6 +258,7 @@ class Model:
             else:
                 # symbol with index to distinguish edge, but no color because the face is not visible
                 return sym + str(idx+1), '?'
+        #FIXME: only for models with 3 axes, function only used with debug flags
         x, y, z = self.cell_indices[blockpos]
         symx, colorsymx = axisidx_to_sym(0, x)
         symy, colorsymy = axisidx_to_sym(1, y)
@@ -252,6 +302,48 @@ class Model:
             mslice = self.sizes[raxis] - 1 - mslice
         return raxis, mslice, rdir
         
+    class MoveParallel (Enum):
+        NO, LT, COMPAT, GT = range(4)
+        
+    def is_parallel(self, iaxis1, slice1, iaxis2, slice2):
+        for idx in range(self.cell_count):
+            pos, rot = idx, ''
+            bslice = self.cell_indices[pos][iaxis1]
+            if bslice != slice1:
+                continue
+            pos, rot = self.rotate_symbolic(iaxis1, False, pos, rot)
+            
+            bslice = self.cell_indices[pos][iaxis2]
+            if bslice != slice2:
+                continue
+            pos, rot = self.rotate_symbolic(iaxis2, False, pos, rot)
+            
+            bslice = self.cell_indices[pos][iaxis1]
+            if bslice == slice1:
+                pos, rot = self.rotate_symbolic(iaxis1, True, pos, rot)
+                
+            bslice = self.cell_indices[pos][iaxis2]
+            if bslice == slice2:
+                pos, rot = self.rotate_symbolic(iaxis2, True, pos, rot)
+                
+            if idx != pos or rot != '':
+                return False
+        else:
+            return True
+        assert False
+        
+    def parallel(self, move1, move2):
+        if move1.axis == move2.axis:
+            if move1.slice == move2.slice:  return self.MoveParallel.COMPAT
+            if move1.slice < move2.slice:   return self.MoveParallel.LT
+            if move1.slice > move2.slice:   return self.MoveParallel.GT
+        if self.is_parallel(move1.axis, move1.slice, move2.axis, move2.slice):
+            if move1.axis < move2.axis: 
+                return self.MoveParallel.LT
+            else:  # move1.axis > move2.axis
+                return self.MoveParallel.GT
+        return self.MoveParallel.NO
+        
     def _get_selected_moves(self, cellidx, vector):
         for iaxis, axis in enumerate(self.axes):
             vda = vector.dot(axis)
@@ -259,19 +351,37 @@ class Model:
                 continue
             mslice = self.cell_indices[cellidx][iaxis]
             if mslice is not None:
+                if not self.is_user_slice(iaxis, mslice):
+                    mslice = -1
                 yield iaxis, mslice, vda
             
+    def _get_other_moves(self, cellidx, vector):
+        '''Moves with axes that are not parallel to vector'''
+        for iaxis, axis in enumerate(self.axes):
+            vca = vector.cross(axis)
+            if all(-epsilon < x < epsilon for x in vca):
+                continue
+            mslice = self.cell_indices[cellidx][iaxis]
+            if mslice is not None:
+                if not self.is_user_slice(iaxis, mslice):
+                    mslice = -1
+                yield iaxis, mslice, vca.normalised()
+                
     def get_selected_moves2(self, cellidx, vert1, vert2):
-        res = sorted(self._get_selected_moves(cellidx, vert2 - vert1), key=lambda m: m[2])
+        res = sorted(self._get_selected_moves(cellidx, vert2 - vert1), key=lambda m: -abs(m[2]))
         a1, s1, d1 = res[0]
         if len(res) == 1:
             return True, (a1, s1, d1>0)
-        a2, s2, d2 = res[-1]
+        a2, s2, d2 = res[1]
+        if s1 >= 0 and s2 < 0:
+            return True, (a1, s1, d1>0)
+        if s1 < 0 and s2 >= 0:
+            return True, (a2, s2, d2>0)
         return False, (a1, s1, d1>0,  a2, s2, d2>0)
         
     def get_selected_move_center(self, cellidx, vector):
         moves = ((ma, ms, md) for ma, ms, md in self._get_selected_moves(cellidx, vector) if ms in [0,self.sizes[ma]-1])
-        ma, ms, md = max(moves, key=lambda m: m[2])
+        ma, ms, md = max(moves, key=lambda m: abs(m[2]))
         return ma, ms, md>0
         
     @staticmethod
@@ -354,51 +464,13 @@ class Model:
         else:
             assert False, count
             
-    def gl_label_vertices(self, polygons, faces, mirror_distance):
-        for *cell_vertices, faceno in polygons:
-            fd = faces[faceno]
-            cell_vertices = [self.vectors[i] for i in cell_vertices]
-            normal = self.vectors[self.normals[faceno]]
-            color = fd.color
-            if fd.imagemode:
-                texrange = self.texranges_mosaic[faceno]
-            else:
-                texrange = get_texcoords_range(cell_vertices, normal)
-            texpos = (get_texcoords(v, normal, *texrange) for v in cell_vertices)
-            x1,y1,x2,y2 = fd.texrect
-            texpos = [(tpx*(x2-x1) + x1, tpy*(y2-y1) + y1) for tpx, tpy in texpos]
-            
-            for (vertex, tp), baryc in self.triangulate_barycentric(
-                                                    len(cell_vertices),
-                                                    zip(cell_vertices, texpos)):
-                yield vertex, normal, color, tp, baryc
-            if mirror_distance is not None:
-                offset = normal * mirror_distance
-                mirror_normal = -normal
-                for (vertex, tp), baryc in self.triangulate_barycentric(
-                                                    len(cell_vertices),
-                                                    zip(cell_vertices, texpos),
-                                                    reverse=True):
-                    yield vertex + offset, mirror_normal, color, tp, baryc
-                    
-    def gl_black_vertices(self, polygons):
-        def vngroup(vns):
-            vns = iter(vns)
-            for v in vns:
-                yield v, next(vns)
-        for vertices_normals in polygons:
-            for ivertex, inormal in self.triangulate(vngroup(vertices_normals)):
-                yield self.vectors[ivertex], self.vectors[inormal]
-                
     def pickarrowdir(self, maxis, mdir, normal):
         if mdir:
             return Vector(self.axes[maxis]).cross(normal).normalised()
         else:
             return normal.cross(Vector(self.axes[maxis])).normalised()
             
-    def _gl_pick_triangles_pickdata_helper(self, cellidx, symbol, normal, vert1, vert2, pickdata):
-        def roundeps(val):
-            return round(val, epsdigits)
+    def _gl_pick_triangles_pickdata_helper(self, cellidx, symbol, normal, vc, vert1, vert2, pickdata):
         one, masd = self.get_selected_moves2(cellidx, vert1, vert2)
         if one:
             maxis, mslice, mdir = masd
@@ -406,22 +478,21 @@ class Model:
             pickdata.append((maxis, mslice, mdir, cellidx, symbol, arrowdir))
         else:
             maxis1, mslice1, mdir1,  maxis2, mslice2, mdir2 = masd
-            sdist1 = roundeps(abs(vert1.dot(self.axes[maxis1]) - self.slice_centers[maxis1][mslice1]))
-            sdist2 = roundeps(abs(vert1.dot(self.axes[maxis2]) - self.slice_centers[maxis2][mslice2]))
-            if sdist1 <= sdist2:
+            vert1_scaled = vc*.1 + vert1*.9 - self.vectors[self.cell_centers[cellidx]]
+            #d = sqrt(v1² + v2² + v3² - (v1*a1 + v2*a2 + v3*a3)²)
+            sdist11_sdist12 = abs(vert1_scaled.dot(self.axes[maxis1])) - abs(vert1_scaled.dot(self.axes[maxis2]))
+            if self.reversepick:
+                sdist11_sdist12 = -sdist11_sdist12
+            if sdist11_sdist12 < 0:
                 arrowdir = self.pickarrowdir(maxis1, mdir1, normal)
                 pickdata.append((maxis1, mslice1, mdir1, cellidx, symbol, arrowdir))
+                arrowdir = self.pickarrowdir(maxis2, mdir2, normal)
+                pickdata.append((maxis2, mslice2, mdir2, cellidx, symbol, arrowdir))
             else:
                 arrowdir = self.pickarrowdir(maxis2, mdir2, normal)
                 pickdata.append((maxis2, mslice2, mdir2, cellidx, symbol, arrowdir))
-            sdist1 = roundeps(abs(vert2.dot(self.axes[maxis1]) - self.slice_centers[maxis1][mslice1]))
-            sdist2 = roundeps(abs(vert2.dot(self.axes[maxis2]) - self.slice_centers[maxis2][mslice2]))
-            if sdist1 < sdist2:
                 arrowdir = self.pickarrowdir(maxis1, mdir1, normal)
                 pickdata.append((maxis1, mslice1, mdir1, cellidx, symbol, arrowdir))
-            else:
-                arrowdir = self.pickarrowdir(maxis2, mdir2, normal)
-                pickdata.append((maxis2, mslice2, mdir2, cellidx, symbol, arrowdir))
         return one
             
     def _gl_pick_triangulate(self, color1, vertsl, normal, mirror_distance):
@@ -450,12 +521,26 @@ class Model:
             for vert in self.triangulate(verts, reverse=True):
                 yield color, vert + mirror_offset
                 
+    def gl_pick_triangles_gesture(self, cellidx, edges_iverts, symbol, normal, mirror_distance, pickdata):
+        # selection_mode: gesture
+        moves = list(self._get_other_moves(cellidx, normal))
+        color = len(pickdata)
+        pickdata.append((None, moves, False, cellidx, symbol, None))
+        verts = [self.vectors[i] for i in edges_iverts]
+        for vert in self.triangulate(verts):
+            yield color, vert
+        if mirror_distance is not None:
+            mirror_offset = normal * mirror_distance
+            for vert in self.triangulate(verts, reverse=True):
+                yield color, vert + mirror_offset
+            
     def gl_pick_triangles_edge3(self, cellidx, edges_iverts, symbol, normal, mirror_distance, pickdata):
         # selection_mode: simple, face is on an edge block of a tetrahedron
         # the first and second vertex are on the edge of the tetrahedron
         vert1, vert2, vert3 = (self.vectors[i] for i in edges_iverts)
+        vc = (vert1+vert2+vert3) / 3
         color1 = len(pickdata)
-        one = self._gl_pick_triangles_pickdata_helper(cellidx, symbol, normal, vert1, vert2, pickdata)
+        one = self._gl_pick_triangles_pickdata_helper(cellidx, symbol, normal, vc, vert1, vert2, pickdata)
         if one:
             vertsl = [[vert1, vert2, vert3]]
         else:
@@ -467,10 +552,11 @@ class Model:
         # selection_mode: simple, face is on an edge block and has 6 vertices, rotates 2 slices
         # the first and second vertex are on the edge
         vert1, vert2, vert3, vert4, vert5, vert6 = (self.vectors[i] for i in edges_iverts)
+        vc = (vert1+vert2+vert3+vert4+vert5+vert6) / 6
         v12 = (vert1 + vert2) / 2
         v45 = (vert4 + vert5) / 2
         color = len(pickdata)
-        one = self._gl_pick_triangles_pickdata_helper(cellidx, symbol, normal, vert1, vert2, pickdata)
+        one = self._gl_pick_triangles_pickdata_helper(cellidx, symbol, normal, vc, vert1, vert2, pickdata)
         if one:
             vertsl = [[vert1, vert2, vert3, vert4, vert5, vert6]]
         else:
@@ -481,8 +567,9 @@ class Model:
         # selection_mode: simple, face is on an edge block, 4 vertices, rotate 1 or 2 slices,
         # the first and second vertex are on the edge
         vert1, vert2, vert3, vert4 = (self.vectors[i] for i in edges_iverts)
+        vc = (vert1+vert2+vert3+vert4) / 4
         color = len(pickdata)
-        one = self._gl_pick_triangles_pickdata_helper(cellidx, symbol, normal, vert1, vert2, pickdata)
+        one = self._gl_pick_triangles_pickdata_helper(cellidx, symbol, normal, vc, vert1, vert2, pickdata)
         if one:
             vertsl = [[vert1, vert2, vert3, vert4]]
         else:
@@ -500,16 +587,17 @@ class Model:
             vert4 = (vert3 + vert1) / 2
         else:
             vert1, vert2, vert3, vert4 = (self.vectors[iv] for iv in edges_iverts)
+        vc = (vert1+vert2+vert3+vert4) / 4
         # first triangle
         color = len(pickdata)
-        one = self._gl_pick_triangles_pickdata_helper(cellidx, symbol, normal, vert1, vert2, pickdata)
+        one = self._gl_pick_triangles_pickdata_helper(cellidx, symbol, normal, vc, vert1, vert2, pickdata)
         if one:
             vertsl = [[vert1, vert2, vert4]]
         else:
             ec12 = (vert1 + vert2) / 2
             vertsl = [[ec12, vert2, vert4], [vert1, ec12, vert4]]
         # second triangle
-        one = self._gl_pick_triangles_pickdata_helper(cellidx, symbol, normal, vert2, vert3, pickdata)
+        one = self._gl_pick_triangles_pickdata_helper(cellidx, symbol, normal, vc, vert2, vert3, pickdata)
         if one:
             vertsl.append([vert2, vert3, vert4])
         else:
@@ -521,15 +609,16 @@ class Model:
         # selection_mode: simple, face is on a corner block, 5 vertices,
         # the second vert is on the corner, the first and third are on the edges
         vert1, vert2, vert3, vert4, vert5 = (self.vectors[iv] for iv in edges_iverts)
+        vc = (vert1+vert2+vert3+vert4+vert5) / 5
         vc13 = (vert1 + vert3) / 2
         ec45 = (vert4 + vert5) / 2
         color1 = len(pickdata)
-        one = self._gl_pick_triangles_pickdata_helper(cellidx, symbol, normal, vert1, vert2, pickdata)
+        one = self._gl_pick_triangles_pickdata_helper(cellidx, symbol, normal, vc, vert1, vert2, pickdata)
         if one:
             vertsl = [[vert1, vert2, ec45, vert5]]
         else:
             vertsl = [[vert1, vert2, vc13], [vert1, vc13, ec45, vert5]]
-        one = self._gl_pick_triangles_pickdata_helper(cellidx, symbol, normal, vert2, vert3, pickdata)
+        one = self._gl_pick_triangles_pickdata_helper(cellidx, symbol, normal, vc, vert2, vert3, pickdata)
         if one:
             vertsl.append([vert2, vert3, vert4, ec45])
         else:
@@ -543,7 +632,7 @@ class Model:
         vert1 = verts[-1]
         for vert2 in verts:
             color1 = len(pickdata)
-            one = self._gl_pick_triangles_pickdata_helper(cellidx, symbol, normal, vert1, vert2, pickdata)
+            one = self._gl_pick_triangles_pickdata_helper(cellidx, symbol, normal, vc, vert1, vert2, pickdata)
             if one:
                 vertsl = [[vc, vert1, vert2]]
             else:
@@ -553,15 +642,19 @@ class Model:
             yield from self._gl_pick_triangulate(color1, vertsl, normal, mirror_distance)
             vert1 = vert2
                 
-    def gl_pick_triangles(self, polygons, selection_mode, mirror_distance, pickdata):
-        for cellidx, (face, picktype), edges_iverts in polygons:
-            symbol = self.faces[face]
-            normal = self.vectors[self.normals[face]]
-            
-            if selection_mode == 0 or picktype == -1:
+    def gl_pick_triangles(self, selection_mode, mirror_distance, pickdata):
+        if DEBUG_NOPICK:
+            return
+        for unused_fvertdata, (normal, cellidx), (fid, picktype, edges_iverts) in self.beveled_fvertdata:
+            if picktype is None:
+                continue
+            symbol = self.faces[fid]
+            if selection_mode == 0:
                 yield from self.gl_pick_triangles_any(cellidx, edges_iverts, symbol, normal, mirror_distance, pickdata)
             elif selection_mode == 1:
-                if picktype == 0:
+                if picktype == -1:
+                    yield from self.gl_pick_triangles_any(cellidx, edges_iverts, symbol, normal, mirror_distance, pickdata)
+                elif picktype == 0:
                     yield from self.gl_pick_triangles_center(cellidx, edges_iverts, symbol, normal, mirror_distance, pickdata)
                 elif picktype == 1:
                     yield from self.gl_pick_triangles_edge3(cellidx, edges_iverts, symbol, normal, mirror_distance, pickdata)
@@ -575,9 +668,141 @@ class Model:
                     yield from self.gl_pick_triangles_edge6(cellidx, edges_iverts, symbol, normal, mirror_distance, pickdata)
                 else:
                     assert False, (selection_mode, picktype)
+            elif selection_mode == 2:
+                yield from self.gl_pick_triangles_gesture(cellidx, edges_iverts, symbol, normal, mirror_distance, pickdata)
             else:
                 assert False, (selection_mode, picktype)
                     
+    def bevel_cell_label(self, cidx, c_faces):
+        # label faces that replace the old faces
+        for hf_face, hf_face_id, hf_verts in c_faces:
+            fvertdata1, (fnormal, fcidx), unused = self.beveled_fvertdata[hf_face]
+            if cidx != fcidx:
+                fnormal = -fnormal
+            yield hf_face_id, fnormal, [fvertdata1[v] for v in hf_verts]
+                
+    def bevel_cell_black(self, cidx, c_faces, c_edges, c_verts):
+        fton = lambda fnormal, fcidx: fnormal if cidx == fcidx else -fnormal
+        if not DEBUG_NOBEVEL:
+            # faces that replace the old verts
+            for v, vertdata in c_verts:
+                vertdata = [(self.beveled_fvertdata[f][0][v], fton(*self.beveled_fvertdata[f][1])) for f in vertdata]
+                yield vertdata
+            # faces that replace the old edges
+            for f1, f3, v1, v3 in c_edges:
+                fvertdata1, finfo, unused = self.beveled_fvertdata[f1]
+                normal1 = fton(*finfo)
+                fvertdata3, finfo, unused = self.beveled_fvertdata[f3]
+                normal3 = fton(*finfo)
+                vertdata = [(fvertdata1[v3], normal1), (fvertdata3[v3], normal3),
+                            (fvertdata3[v1], normal3), (fvertdata1[v1], normal1)]
+                yield vertdata
+        # black faces that replace the old faces
+        for hf_face, hf_verts in c_faces:
+            fvertdata1, finfo, unused = self.beveled_fvertdata[hf_face]
+            normal1 = fton(*finfo)
+            yield [(fvertdata1[v], normal1) for v in hf_verts]
+            
+    def gl_label_vertices(self, facesdata, faces, mirror_distance):
+        if DEBUG_NOLABEL:
+            return
+        for faceno, normal, vertices in facesdata:
+            fd = faces[faceno]
+            color = fd.color
+            if fd.imagemode:
+                texrange = self.texranges_mosaic[faceno]
+            else:
+                texrange = get_texcoords_range(vertices, normal)
+            texpos = (get_texcoords(v, normal, *texrange) for v in vertices)
+            x1,y1,x2,y2 = fd.texrect
+            texpos = [(tpx*(x2-x1) + x1, tpy*(y2-y1) + y1) for tpx, tpy in texpos]
+            
+            for (vertex, tp), baryc in self.triangulate_barycentric(
+                                                    len(vertices),
+                                                    zip(vertices, texpos)):
+                yield vertex, normal, color, tp, baryc
+            if mirror_distance is not None:
+                offset = normal * mirror_distance
+                mirror_normal = -normal
+                for (vertex, tp), baryc in self.triangulate_barycentric(
+                                                    len(vertices),
+                                                    zip(vertices, texpos),
+                                                    reverse=True):
+                    yield vertex + offset, mirror_normal, color, tp, baryc
+                    
+    def gl_black_vertices(self, facesdata):
+        if DEBUG_NOBLACK:
+            return
+        for vertices in facesdata:
+            for ivertex, normal in self.triangulate(vertices):
+                yield ivertex, normal
+                
+    def gl_bevel_faces(self):
+        width = self.bevel_width
+        if width == 0:
+            def bevel_points(ps):
+                return {p: p for p in ps}
+        else:
+            def bevel_points(ps):
+                def bevel_point(point1, point2, point3):
+                    point1 = self.vectors[point1]
+                    point2 = self.vectors[point2]
+                    point3 = self.vectors[point3]
+                    nd1 = (point1 - point2).normalised()
+                    nd2 = (point3 - point2).normalised()
+                    return point2 + (nd1 + nd2) * width / nd1.cross(nd2).length()
+                lps = len(ps)
+                return {p: bevel_point(ps[(i-1)%lps], p, ps[(i+1)%lps]) for i, p in enumerate(ps)}
+        def face_normal(p1, p2, p3, *unused_args):
+            p1 = self.vectors[p1]
+            p2 = self.vectors[p2]
+            p3 = self.vectors[p3]
+            return (p3 - p2).cross(p1 - p2).normalised()
+        return [(bevel_points(ps), (face_normal(*ps), cidx), (fid, picktype, ps))
+                        for cidx, cfd in enumerate(self.facesdata) for fid, picktype, ps in cfd]
+                    
+    def gl_debug_vertices(self, faces, mirror_distance):
+        # yield vertices, colors
+        # normals
+        if DEBUG_DRAWNORMALS:
+            for cidx, (c_faces_label, c_faces_black, c_edges, c_verts) in enumerate(self.blocksdata):
+                for faceno, normal, vertices in self.bevel_cell_label(cidx, c_faces_label):
+                    fd = faces[faceno]
+                    color = fd.color
+                    
+                    vertex = sum(vertices, Vector()) / len(vertices)
+                    yield vertex, [0,0,0]
+                    yield vertex+normal, color
+                    if mirror_distance is not None:
+                        offset = normal * mirror_distance
+                        yield vertex+offset, [0,0,0]
+                        yield vertex+offset-normal, color
+        # axes
+        if DEBUG_DRAWAXES:
+            for i, axis in enumerate(self.axes):
+                symbol = self.symbols[i]
+                faceno = self.faces.index(symbol)
+                fd = faces[faceno]
+                color = fd.color
+                vaxis = Vector(axis)*self.sizes[i]
+                yield -vaxis, [0,0,0]
+                yield -vaxis+axis, color
+                yield vaxis, [0,0,0]
+                yield vaxis+axis, color
+        # coordinates
+        x = float(-self.sizes[0])
+        y = float(-self.sizes[1])
+        z = float(-self.sizes[2])
+        yield [x-1, y-1, z,  -x,  y-1,  z], [255, 0, 0,  255, 0, 0]
+        yield [x-1, y-1, z,  x-1, -y,   z], [0, 255, 0,  0, 255, 0]
+        yield [x-1, y-1, z,  x-1, y-1, -z], [0, 0, 255,  0, 0, 255]
+        
+    def gl_debug_vertices_2D(self):
+        # selection (modelview)
+        yield [0, 0, 0,  1, 1, 1], [255, 255, 255,  255, 255, 255]
+        # selection (viewport)
+        yield [0, 0, 0,  1, 1, 0], [255, 255, 0,  255, 0, 0]
+        
     def gl_vertex_data(self, selection_mode, theme, mirror_distance):
         vertices = []
         normals = []
@@ -588,10 +813,14 @@ class Model:
         cnts_block = []
         faces = [theme.faces[fk] for fk in self.facekeys]
         
-        for label_polygons, black_polygons in self.block_polygons:
+        for cidx, (c_faces_label, c_faces_black, c_edges, c_verts) in enumerate(self.blocksdata):
+            # bevel
+            beveled_facesdata_label = self.bevel_cell_label(cidx, c_faces_label)
+            beveled_facesdata_black = self.bevel_cell_black(cidx, c_faces_black, c_edges, c_verts)
+            
             cnt = 0
             # label vertices
-            for v, n, c, t, b in self.gl_label_vertices(label_polygons, faces, mirror_distance):
+            for v, n, c, t, b in self.gl_label_vertices(beveled_facesdata_label, faces, mirror_distance):
                 vertices += v
                 normals += n
                 colors += c
@@ -599,7 +828,7 @@ class Model:
                 baryc += b
                 cnt += 1
             # bevel, hidden labels
-            for v, n in self.gl_black_vertices(black_polygons):
+            for v, n in self.gl_black_vertices(beveled_facesdata_black):
                 vertices += v
                 normals += n
                 #TODO: colors and texpos unused
@@ -610,27 +839,18 @@ class Model:
             cnts_block.append(cnt)
         
         idx_debug = len(vertices) // 3
+        cnt_debug = 0
         if DEBUG_DRAW:
-            #FIXME: what if len(self.sizes) < 3 ?
-            x = float(-self.sizes[0])
-            y = float(-self.sizes[1])
-            z = float(-self.sizes[2])
-            # axes
-            vertices += [x-1, y-1, z,  -x, y-1, z]
-            colors += [255, 0, 0,  255, 0, 0]
-            vertices += [x-1, y-1, z,  x-1, -y, z]
-            colors += [0, 255, 0,  0, 255, 0]
-            vertices += [x-1, y-1, z,  x-1, y-1, -z]
-            colors += [0, 0, 255,  0, 0, 255]
-            # selection (modelview)
-            vertices += [0, 0, 0,  1, 1, 1]
-            colors += [255, 255, 255,  255, 255, 255]
-            # selection (viewport)
-            vertices += [0, 0, 0,  1, 1, 0]
-            colors += [255, 255, 0,  255, 0, 0]
-            
             assert len(cnts_block) == self.cell_count
             assert sum(cnts_block) == idx_debug
+            for v, c in self.gl_debug_vertices(faces, mirror_distance):
+                vertices += v
+                colors += c
+            cnt_debug = len(vertices) // 3 - idx_debug
+            for v, c in self.gl_debug_vertices_2D():
+                vertices += v
+                colors += c
+            assert len(vertices) == len(colors)
             
         pickvertices, pickcolors, pickdata = self.gl_pick_data(selection_mode, mirror_distance)
         cnt_pick = len(pickvertices) // 3
@@ -656,7 +876,7 @@ class Model:
         return (    self.cell_count,
                     vertexdata,
                     (ptrnormals, ptrcolors, ptrtexpos, ptrbarycentric, ptrpickvertices, ptrpickcolors),
-                    (cnts_block, idx_debug, cnt_pick),
+                    (cnts_block, idx_debug, cnt_debug, cnt_pick),
                     self.rotation_matrices,
                ), pickdata
                 
@@ -665,10 +885,9 @@ class Model:
         vertices = []
         colors = []
         pickdata = [()]  # list of (maxis, mslice, mdir, face, center, cellidx, symbol, arrowdir)
-        for col, v in self.gl_pick_triangles(self.pick_polygons, selection_mode, mirror_distance, pickdata):
+        for col, v in self.gl_pick_triangles(selection_mode, mirror_distance, pickdata):
             vertices += v
-            color = [(col>>4) & 0xf0, (col) & 0xf0, (col<<4) & 0xf0]
-            colors += color
+            colors += ((col>>4) & 0xf0, (col) & 0xf0, (col<<4) & 0xf0)
         assert len(vertices) == len(colors)
         return vertices, colors, pickdata
         
