@@ -1,6 +1,6 @@
 #-*- coding:utf-8 -*-
 
-#  Copyright © 2009-2015  B. Clausius <barcc@gmx.de>
+#  Copyright © 2009-2015, 2017  B. Clausius <barcc@gmx.de>
 #
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -29,49 +29,37 @@ class MoveData (namedtuple('_MoveData', 'axis slice dir')):     # pylint: disabl
     __slots__ = ()
     def inverted(self):
         return self._replace(dir=not self.dir)  # pylint: disable=E1101
+        
 
-class MoveDataPacked (namedtuple('_MoveDataPacked', 'axis counts fullcount')):     # pylint: disable=W0232
+def normalized_dirs(model, axis, dirs):
+    symmetry = model.symmetries[axis]
+    dirs = dirs % symmetry
+    if dirs > symmetry // 2:
+        dirs -= symmetry
+    return dirs
+    
+class MoveDataPacked (namedtuple('_MoveDataPacked', 'axis slice dirs')):
     # axis  = 0...
-    # counts = [...],  length: model.sizes[axis], values: number of slice-rotations, >0 CW <0 CCW
-    # fullcount = number of full rotations, >0 CW <0 CCW
+    # slice = 0...dim-1, (-1 for all slices not allowed in some functions)
+    # dirs = -symmetry/2...+symmetry/2, >0 CW <0 CCW
     __slots__ = ()
     
+    def __new__(self, maxis, mslice, mdirs, model=None):
+        if model is not None:
+            mdirs = normalized_dirs(model, maxis, mdirs)
+        return super().__new__(self, maxis, mslice, mdirs)
+        
     def length(self):
-        return sum(abs(c) for c in self.counts)
+        return abs(self.dirs)
         
-    def get_full_moves(self, model):
-        symmetry = model.symmetries[self.axis]
-        vals = [0] * symmetry
-        for i, scount in enumerate(self.counts):
-            for s in range(symmetry):
-                v = scount + s
-                if v > symmetry//2:
-                    v -= symmetry
-                vals[s] += abs(v)
-        minval = vals[0]+1
-        for i, v in enumerate(vals):
-            if v < minval:
-                f = i
-                minval = v
-        f = symmetry - f + self.fullcount
-        if f > symmetry//2:
-            f -= symmetry
-        return f
-        
-    def update_fullmoves(self, model):
-        def counts_offset(fullcount):
-            symmetry = model.symmetries[self.axis]
-            for sdirs in self.counts:
-                sdirs -= fullcount
-                if sdirs < 0:
-                    sdirs += symmetry
-                if sdirs > symmetry // 2:
-                    sdirs -= symmetry
-                yield sdirs
-        fullcount = self.get_full_moves(model)
-        counts = list(counts_offset(fullcount))
-        return self._replace(counts=counts, fullcount=fullcount)
+    def add_dirs(self, dirs, model):
+        return self._replace(dirs=normalized_dirs(model, self.axis, self.dirs+dirs))
         
+    def unpack(self):
+        mdir = self.dirs < 0
+        for unused in range(self.length()):
+            yield self.axis, self.slice, mdir
+            
         
 class _MoveQueueBase:
     class MoveQueueItem:
@@ -95,17 +83,6 @@ class _MoveQueueBase:
         def normalize_complete_moves(cls, model, totalmoves):
             for move in model.normalize_complete_moves(move.data for move in totalmoves):
                 yield cls(MoveData(*move))
-        def unpack(self):
-            maxis, sslices, fullmoves = self.data
-            # slice rotations
-            for mslice, sdirs in enumerate(sslices):
-                mdir = sdirs < 0
-                for unused in range(abs(sdirs)):
-                    yield maxis, mslice, mdir
-            # total rotations
-            mdir = fullmoves < 0
-            for unused in range(abs(fullmoves)):
-                yield maxis, -1, mdir
         def update_fullmoves(self, model):
             self.data = self.data.update_fullmoves(model)
             
@@ -225,25 +202,26 @@ class MoveQueue (_MoveQueueBase):
         totalmoves = list(self.MoveQueueItem.normalize_complete_moves(model, totalmoves))
         self.moves = new_moves + totalmoves
         
-    def unpack(self, move):
-        for move in move.unpack():
+    def push_packed(self, pmove):
+        for move in pmove.data.unpack():
             self.push(move)
         
     def normalize_moves(self, model):
         mqp = MoveQueuePacked()
+        mqp_current_place = (0, 0)
         for i, move in enumerate(self.moves):
             if i == self.current_place:
-                mqp.current_place = (len(mqp.moves)-1, mqp.moves[-1].data.length()) if mqp.moves else (0, 0)
-            mqp.pack(move.data, model)
+                mqp_current_place = (len(mqp.moves)-1, mqp.moves[-1].data.length()) if mqp.moves else (0, 0)
+            mqp.push_unpacked(move.data, model)
+        mqp_moves = mqp.separate_full_moves(model)
         if len(self.moves) <= self.current_place:
-            mqp.current_place = len(mqp.moves), 0
+            mqp_current_place = len(mqp_moves), 0
         self.reset()
-        for i, move in enumerate(mqp.moves):
-            move.update_fullmoves(model)
-            if i == mqp.current_place[0]:
-                self.current_place = len(self.moves) + mqp.current_place[1]
-            self.unpack(move)
-        if len(mqp.moves) <= mqp.current_place[0]:
+        for i, move in enumerate(mqp_moves):
+            if i == mqp_current_place[0]:
+                self.current_place = len(self.moves) + mqp_current_place[1]
+            self.push_packed(move)
+        if len(mqp_moves) <= mqp_current_place[0]:
             self.current_place = len(self.moves)
         
     def is_mark_current(self):
@@ -323,47 +301,133 @@ class MoveQueuePacked (_MoveQueueBase):
             i += move.data.length()
         return i
         
-    def push(self, move_data, **kwargs):
-        new_element = self.MoveQueueItem(MoveDataPacked(*move_data), **kwargs)
-        self.moves.append(new_element)
+    def axis_length(self, axis_dirs, axis, model):
+        symmetry = model.symmetries[axis]
+        if model.ignored_slice[axis] is None:
+            vals = [0] * symmetry
+            for m_dirs in axis_dirs:
+                for s_base in range(symmetry):
+                    v = m_dirs + s_base
+                    if v > symmetry//2:
+                        v -= symmetry
+                    vals[s_base] += abs(v)
+            minval = vals[0]+1
+            for i, v in enumerate(vals):
+                if v < minval:
+                    f = i
+                    minval = v
+            f = symmetry - f
+            if f > symmetry//2:
+                f -= symmetry
+            return minval, f
+        else:
+            f = axis_dirs[model.ignored_slice[axis]]
+            length = 0
+            for m_dirs in axis_dirs:
+                v = m_dirs - f + symmetry
+                if v > symmetry//2:
+                    v -= symmetry
+                length += abs(v)
+            return length, f
+            
+    def length_no_full(self, model):
+        i = 0
+        axis = None
+        axis_dirs = None
+        for move in self.moves:
+            if axis != move.data.axis:
+                if axis is not None:
+                    i += self.axis_length(axis_dirs, axis, model)[0]
+                axis = move.data.axis
+                axis_dirs = [0] * model.sizes[axis]
+            assert move.data.slice >= 0
+            assert axis_dirs[move.data.slice] == 0
+            axis_dirs[move.data.slice] = move.data.dirs
+        if axis is not None:
+            i += self.axis_length(axis_dirs, axis, model)[0]
+        return i
         
-    def pack(self, move, model):
-        def getstack():
-            if self.moves:
-                return self.moves[-1].data
-            return None
-        def newsmove(maxis):
-            return MoveDataPacked(maxis, [0] * model.sizes[maxis], 0)
-        def addslice(scounts, move):
-            sdir = -1 if move.dir else 1
-            if move.slice < 0:
-                for s in range(model.sizes[move.axis]):
-                    scounts[s] += sdir
-            else:
-                scounts[move.slice] += sdir
-        def symmetry(smove):
-            ssymmetry = model.symmetries[smove.axis]
-            for i, scount in enumerate(smove.counts):
-                if scount < 0:
-                    scount += ssymmetry
-                if scount > ssymmetry//2:
-                    scount -= ssymmetry
-                smove.counts[i] = scount
-                
-        smove = getstack()
-        if smove is None or smove.axis != move.axis:
-            smove = newsmove(move.axis)
-            self.push(smove)
-        addslice(smove.counts, move)
-        symmetry(smove)
-        if not any(smove.counts) and not smove.fullcount:
-            self.moves.pop()
-        return smove
+    def _separate_full_moves(self, i_axis, j_axis, axis, fullmoves, model):
+        if not fullmoves:
+            return self.moves[i_axis:j_axis]
+        mslice = 0
+        newmoves = []
+        for i in range(i_axis, j_axis):
+            move = self.moves[i]
+            for j in range(mslice, move.data.slice):
+                moveitem = self.MoveQueueItem(MoveDataPacked(axis, j, -fullmoves, model))
+                newmoves.append(moveitem)
+            assert axis == move.data.axis
+            move.data = move.data.add_dirs(-fullmoves, model)
+            newmoves.append(move)
+            mslice = move.data.slice + 1
+        for j in range(mslice, model.sizes[axis]):
+            moveitem = self.MoveQueueItem(MoveDataPacked(axis, j, -fullmoves, model))
+            newmoves.append(moveitem)
+        moveitem = self.MoveQueueItem(MoveDataPacked(axis, -1, fullmoves))
+        newmoves.append(moveitem)
+        return newmoves
+        
+    def separate_full_moves(self, model):
+        i_axis = 0
+        axis = None
+        axis_dirs = None
+        newmoves = []
+        for i, move in enumerate(self.moves):
+            if axis != move.data.axis:
+                if axis is not None:
+                    fullmoves = self.axis_length(axis_dirs, axis, model)[1]
+                    newmoves += self._separate_full_moves(i_axis, i, axis, fullmoves, model)
+                i_axis = i
+                axis = move.data.axis
+                axis_dirs = [0] * model.sizes[axis]
+            assert move.data.slice >= 0
+            assert axis_dirs[move.data.slice] == 0, (axis_dirs, i, move.data.slice)
+            axis_dirs[move.data.slice] = move.data.dirs
+        if axis is not None:
+            fullmoves = self.axis_length(axis_dirs, axis, model)[1]
+            newmoves += self._separate_full_moves(i_axis, len(self.moves), axis, fullmoves, model)
+        return newmoves
+        
+    def push_unpacked(self, move, model):
+        # this function assumes that self.moves does not contain full moves,
+        if move.slice < 0:
+            # resolve full moves to slice moves
+            for i in range(model.sizes[move.axis]):
+                self.push_unpacked(MoveData(move.axis, i, move.dir), model)
+            return
+            
+        i = len(self.moves)
+        while True:
+            if i <= 0:
+                moveitem = None
+                break
+            # get item
+            _moveitem = self.moves[i-1]
+            parallel = model.parallel(_moveitem.data, move)
+            if parallel == parallel.NO or parallel == parallel.LT:
+                moveitem = None
+                break
+            moveitem = _moveitem
+            i -= 1
+            if parallel == parallel.COMPAT:
+                break
+            assert parallel == parallel.GT
+        # create new empty item if needed
+        if moveitem is None:
+            moveitem = self.MoveQueueItem(MoveDataPacked(move.axis, move.slice, 0))
+            self.moves.insert(i, moveitem)
+        # combine move with the compatible moveitem.data
+        assert move.axis == moveitem.data.axis
+        moveitem.data = moveitem.data.add_dirs((-1 if move.dir else 1), model)
+        # remove if empty
+        if moveitem.data.dirs == 0:
+            self.moves.pop(i)
             
     def format(self, model):
         mq = MoveQueue()
         for move in self.moves:
-            mq.unpack(move)
+            mq.push_packed(move)
         return mq.format(model)
         
                 
@@ -405,7 +469,6 @@ class MoveFormat: # pylint: disable=W0232
     @classmethod
     def format(cls, move, model):
         mface, mslice, mdir = cls.intern_to_str_move(move, model)
-        #print('format:', move.data, '->', (mface, mslice, mdir))
         mark = ' ' if move.mark_after else ''
         move_code = mface + mslice + mdir + mark
         return move_code
@@ -433,10 +496,9 @@ class MoveFormat: # pylint: disable=W0232
         
     @classmethod
     def parse(cls, move_code, model):
-        tface, tslice, tdir, err1, mark, err2 = cls.re_flubrd.match(move_code).groups()
+        tface, tslice, tdir, unused_err1, mark, unused_err2 = cls.re_flubrd.match(move_code).groups()
         mark = bool(mark)
         move = cls.str_to_intern_move(tface, tslice, tdir, model)
-        #print('parse:', (tface, tslice, tdir), '->', move)
         if move is None:
             debug('Error parsing formula')
             return move, False
