@@ -1,6 +1,6 @@
 #-*- coding:utf-8 -*-
 
-#  Copyright © 2009, 2011-2015  B. Clausius <barcc@gmx.de>
+#  Copyright © 2009, 2011-2017  B. Clausius <barcc@gmx.de>
 #
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@ import sys
 import importlib
 from collections import namedtuple, OrderedDict
 
-from .debug import debug, DEBUG_ALG
+from .debug import debug, DEBUG_MSG, DEBUG_ALG, DEBUG_LIVEPLUGINS
 from . import config
 from . import model
 
@@ -61,9 +61,9 @@ class PluginFileLoader:
         self.parse()
         
     def load(self, filename):
+        debug('loading plugin-file', repr(filename))
         with open(filename, 'rt', encoding='utf-8') as fp:
             lines = fp.readlines()
-        debug('Loaded plugin:', filename)
         
         # parse file
         para = {}
@@ -102,13 +102,13 @@ class PluginFileLoader:
             self.paras.append(para)
         
     @staticmethod
-    def parse_version(value, lineno, comment):
+    def parse_version(value, unused_lineno, unused_comment):
         if value != FILE_FORMAT:
             raise PluginFileError('wrong file version:', value)
         return value
         
     @staticmethod
-    def parse_model(modelstrings, lineno, comment):
+    def parse_model(modelstrings, unused_lineno, unused_comment):
         model_infos = []
         for modelstr in modelstrings or []:
             try:
@@ -120,7 +120,7 @@ class PluginFileLoader:
         return model_infos
         
     @staticmethod
-    def parse_refblocks(value, lineno, comment):
+    def parse_refblocks(value, unused_lineno, unused_comment):
         return value and [[v, v] for v in value.upper().split()]
         
     @staticmethod
@@ -158,7 +158,7 @@ class PluginFileLoader:
         return value, tuple(_translate(value)), lineno, comment
         
     @staticmethod
-    def parse_solution(value, lineno, comment):
+    def parse_solution(value, unused_lineno, unused_comment):
         def split_solution(value):
             try:
                 conditions, moves = value.split(',')
@@ -173,7 +173,7 @@ class PluginFileLoader:
         return value and [split_solution(v) for v in value if v]
         
     @staticmethod
-    def parse_moves(value, lineno, comment):
+    def parse_moves(value, unused_lineno, unused_comment):
         if value is None:
             return None
         moves = ' '.join(value)
@@ -184,7 +184,7 @@ class PluginFileLoader:
         return func
         
     @staticmethod
-    def parse_module(value, lineno, comment):
+    def parse_module(value, unused_lineno, unused_comment):
         if value is None:
             return None
         value = value.split(',')
@@ -260,7 +260,6 @@ class PluginFileLoader:
 class PluginHelper:
     def __init__(self):
         self.scripts = OrderedDict()
-        self.error_messages = []
         
     @staticmethod
     def test_model(model, models):
@@ -299,17 +298,18 @@ class PluginHelper:
             # all model rejected
             all_models = [model_[0] for model_ in all_models if model_[1] is not None]
             if not all_models:
-                raise PluginModelCompatError(_('This plugin does not work for any model.\n'))
+                raise PluginModelCompatError(_('This plugin does not work for any model.')+'\n')
             elif len(all_models) == 1:
-                raise PluginModelCompatError(_('This plugin only works for:\n') + '  ' + all_models[0])
+                raise PluginModelCompatError(_('This plugin only works for:') + '\n  ' + all_models[0])
             else:
                 raise PluginModelCompatError(
-                        _('This plugin only works for:\n') +
+                        _('This plugin only works for:') + '\n' +
                         '\n'.join([' • ' + m for m in all_models])
                     )
         
     def load_plugins_from_directory(self, dirname):
-        debug('Loading Plugins from', dirname)
+        if DEBUG_MSG or DEBUG_LIVEPLUGINS:
+            print('loading plugins from', repr(dirname))
         for filename in sorted(os.listdir(dirname), key=str.lower):
             unused_name, ext = os.path.splitext(filename)
             if ext != '.plugins':
@@ -317,14 +317,12 @@ class PluginHelper:
             if DEBUG_ALG: debug('  plugins:', filename)
             try:
                 self.load_plugins_from_file(os.path.join(dirname, filename))
-            except Exception as e:      # pylint: disable=W0703
-                self.error_messages.append('Error loading {}:\n{}'.format(os.path.basename(filename), e))
+            except Exception:
                 sys.excepthook(*sys.exc_info())
         
     def load_plugins(self):
         self.scripts.clear()
-        del self.error_messages[:]
-        for dirname in [config.PLUGINS_DIR, config.USER_PLUGINS_DIR]:
+        for dirname in [config.PLUGINS_DIR, config.get_data_home(config.PACKAGE, 'plugins')]:
             if os.path.isdir(dirname):
                 debug('Found plugins path:', dirname)
                 self.load_plugins_from_directory(dirname)
