@@ -1,7 +1,7 @@
 #!/usr/bin/python3
 #-*- coding:utf-8 -*-
 
-#  Copyright © 2009, 2011-2015  B. Clausius <barcc@gmx.de>
+#  Copyright © 2009, 2011-2015, 2017  B. Clausius <barcc@gmx.de>
 #
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@ import pybiklib.main
 root_dir = os.path.dirname(os.path.realpath(os.path.dirname(__file__)))
 
 def main():
-    pybiklib.main.run(root_dir)
+    pybiklib.main.Main(root_dir)
 
 if __name__ == '__main__':
     main()
